@@ -7,17 +7,13 @@ reuse.
 
 """
 from numbers import Real
-from typing import Any
-from typing import Iterable
-from typing import Tuple
+from typing import Any, Iterable, Tuple
 
 import numpy as np
 
-from .format_str_case import format_str_case
-from .format_str_case import SUPPORTED_STR_FORMAT_OPTIONS
+from .format_str_case import SUPPORTED_STR_FORMAT_OPTIONS, format_str_case
 from .options import Options
-from .utils import format_for_output
-from .utils import generate_name_description_error_message
+from .utils import format_for_output, generate_name_description_error_message
 
 __all__ = ["processed_property"]
 
@@ -29,14 +25,31 @@ def processed_property(name, **kwargs):
     ----------
     name : str
         Attribute name that will be used for the property.
-    **kwargs
-        Arbitrary keyword arguments.
+    read_only : bool, False
+        If a processed property is specified as read-only then after its value
+        has been set once (i.e. after it has been instantiated), it is not able
+        to be set again.
+    type : type, None
+        Tells the processed property the type of the value that the setter is
+        expecting. If the value passed to the setter is not of this type then a
+        TypeError will be raised.
+    cast : bool, False
+        Defines whether the value passed to the processed property's setter
+        should be attempted to be cast to the expected type.
+    optional : bool, False
+        In additional to other specifications for the processed property, a
+        value of `None` is also accepted.
+    default : obj, None
+        If a processed property is specified as `optional`, then if a value of
+        `None` is supplied to the setter this is instead replaced with this
+        specified default value.
 
     Returns
     -------
     property
         The metaprogrammed property object with setter including specified
         settings.
+
     """
 
     def parse_kwarg(kwarg_name, description, valids, default):
@@ -141,7 +154,8 @@ def processed_property(name, **kwargs):
             args = (name_str, )
             setter_dispatcher.update({process_optimisable: (args, {})})
         if post_method is not None:
-            setter_dispatcher.update({apply_method: no_args_kwargs})
+            args = (optional, post_method)
+            setter_dispatcher.update({apply_method: (args, {})})
         return setter_dispatcher
 
     storage_name = "_" + name
@@ -150,7 +164,7 @@ def processed_property(name, **kwargs):
     options = kwargs.get("options", None)
     unsupported_options = kwargs.get("unsupported_options", [])
     optional = kwargs.get("optional", False)
-    default = kwargs.get("default", False)
+    default = kwargs.get("default", None)
     iterable_allowed = kwargs.get("iterable_allowed", False)
     cast_to_type = kwargs.get("cast", False)
     len_sequence = kwargs.get("len")
@@ -230,12 +244,14 @@ def check_expected_type(value, iterable_allowed, expected_type, name_str,
                 check_type(val, expected_type, name_str, optional,
                            cast_to_type, default) for val in value
             ])
-        else:
-            value = (check_type(value, expected_type, name_str, optional,
-                                cast_to_type, default), )
-    else:
-        value = check_type(value, expected_type, name_str, optional,
-                           cast_to_type, default)
+        elif (value is None) and optional:
+            if default is not None:
+                return default
+            return None
+        value = (check_type(value, expected_type, name_str, optional,
+                            cast_to_type, default), )
+    value = check_type(value, expected_type, name_str, optional,
+                       cast_to_type, default)
     return value
 
 
@@ -265,20 +281,18 @@ def check_type(value, expected_type, name_str, optional, cast_to_type,
     """
     if isinstance(value, expected_type):
         return value
-    elif optional and value is None:
-        if default:
+    elif optional and (value is None):
+        if default is not None:
             return default
-        else:
-            return None
+        return None
     elif cast_to_type:
-        return cast_type(value, expected_type)
-    else:
-        msg = (f"{name_str} must be a {repr(expected_type)}, instead got "
-               f"a {repr(type(value))}.")
-        raise TypeError(msg)
+        return cast_type(value, expected_type, name_str)
+    msg = (f"{name_str} must be a {repr(expected_type)}, instead got "
+           f"a {repr(type(value))}.")
+    raise TypeError(msg)
 
 
-def cast_type(value, expected_type):
+def cast_type(value, expected_type, name_str):
     """Enforce type casting of property value to be set to specific type.
 
     Parameters
@@ -304,10 +318,10 @@ def cast_type(value, expected_type):
     cast_str = f"processed_value = {expected_type.__name__}({value})"
     try:
         exec(cast_str)
-    except (ValueError, TypeError) as e:
+    except (ValueError, TypeError):
         msg = (f"{name_str} must be a {repr(expected_type)}, instead got "
                f"a {repr(type(value))} which cannot be cast.")
-        raise e(msg)
+        raise ValueError(msg)
     return locals()["processed_value"]
 
 
@@ -521,7 +535,7 @@ def check_len(value, len_sequence, name_str):
     return value
 
 
-def apply_method(value):
+def apply_method(value, optional, post_method):
     """Applies a specified method at the end of the property setter.
 
     Parameters
